<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Observer;

use FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper;
use Magento\Catalog\Api\CategoryLinkManagementInterface;
use Magento\Catalog\Api\Data\ProductAttributeInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Eav\Model\Config as EavConfig;
use Magento\Framework\App\{ResourceConnectionFactory, ResourceConnection};
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\LocalizedException;
use Zend_Db;

/**
 * Class AfterCatalogProductSaveEntityObserver
 * @package FiloBlu\Refilo\Observer
 */
class AfterCatalogProductSaveEntityObserver implements ObserverInterface
{
    /**
     * @var ProductRepository
     */
    private $productRepository;
    /**
     * @var ResourceConnectionFactory
     */
    private $resourceConnectionFactory;
    /**
     * @var EavConfig
     */
    private $eavConfig;
    /**
     * @var ProductAttributeHelper
     */
    private $productAttributeHelper;
    /**
     * @var CategoryLinkManagementInterface
     */
    private $categoryLinkManagement;

    /**
     * @param ResourceConnectionFactory $resourceConnectionFactory
     * @param EavConfig $eavConfig
     * @param ProductAttributeHelper $productAttributeHelper
     * @param ProductRepository $productRepository
     * @param CategoryLinkManagementInterface $categoryLinkManagement
     */
    public function __construct(
        ResourceConnectionFactory $resourceConnectionFactory,
        EavConfig $eavConfig,
        ProductAttributeHelper $productAttributeHelper,
        ProductRepository $productRepository,
        CategoryLinkManagementInterface $categoryLinkManagement
    ) {
        $this->resourceConnectionFactory = $resourceConnectionFactory;
        $this->eavConfig = $eavConfig;
        $this->productAttributeHelper = $productAttributeHelper;
        $this->productRepository = $productRepository;
        $this->categoryLinkManagement = $categoryLinkManagement;
    }

    /**
     * @param Observer $observer
     * @return $this
     * @throws LocalizedException
     */
    public function execute(Observer $observer)
    {
        if (!$product = $observer->getData('product')) {
            return $this;
        }

        $productSku = $product->getSku();
        $originalProductCategoryIds = $product->getData('category_ids');

        $prodTypeId = $this->eavConfig
            ->getEntityType(ProductAttributeInterface::ENTITY_TYPE_CODE)
            ->getEntityTypeId();

        /** @var ResourceConnection $resourceConnection */
        $resourceConnection = $this->resourceConnectionFactory->create();
        $eavAttributeTable = $resourceConnection->getTableName('eav_attribute');
        $catalogEavAttribute = $resourceConnection->getTableName('catalog_eav_attribute');

        $sql = "SELECT
                    ea.attribute_code
                FROM
                    {$eavAttributeTable} ea
                LEFT JOIN {$catalogEavAttribute} cea ON
                    ea.attribute_id = cea.attribute_id
                WHERE ea.entity_type_id = {$prodTypeId} AND cea.is_pagebuilder_enabled = 1";

        $pageBuilderAttributes = $resourceConnection
            ->getConnection()
            ->fetchAll($sql, [], Zend_Db::FETCH_ASSOC);

        if (count($pageBuilderAttributes)) {
            foreach ($pageBuilderAttributes as $attribute) {
                $attributeCode = $attribute['attribute_code'];
                if ($product->getData($attributeCode) !== null && !is_bool($product[$attributeCode])) {
                    $product->setData($attributeCode, $this->productAttributeHelper->checkHtmlAttribute($product[$attributeCode]));
                }
            }

            $this->productRepository->save($product);

            $this->categoryLinkManagement->assignProductToCategories(
                $productSku,
                $originalProductCategoryIds
            );
        }

        return $this;
    }
}
